import axios from 'axios'
import { Message, Notification } from 'element-ui'
// Progress 进度条
import NProgress from 'nprogress'
// Progress 进度条样式
import 'nprogress/nprogress.css'
// 禁止显示圆环
NProgress.configure({ showSpinner: false })
const CONFIG = window.CONFIG

function toFormData(data) {
  return data instanceof Object ? Object.entries(data).map(e => `${encodeURIComponent(e[0])}=${e[1] !== undefined ? encodeURIComponent(e[1]) : ''}`).join('&') : data
}
function checkStatus(status, currentStatus) {
  if (status === undefined || status === null) {
    return false
  } else if (Array.isArray(status)) {
    return !status.includes(currentStatus)
  } else {
    return status !== currentStatus
  }
}

// 取消请求
const CancelToken = axios.CancelToken
const source = CancelToken.source()
// 创建axios实例
const request = axios.create({
  _method: 'post',
  timeout: 20000,
  encrypt: CONFIG.ENCRYPT,
  baseURL: CONFIG.BASE_URL,
  cancelToken: source.token,
  headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
  validateStatus(status) {
    if (status !== 200) {
      Message({
        type: 'error',
        message: `${status} : 请求异常`
      })
    }
    // default
    return status >= 200 && status < 300
  }
})

// 进度条request拦截器
request.interceptors.request.use(config => {
  NProgress.start()
  return config
}, err => {
  NProgress.done()
  Promise.reject(err)
})

// 下载request拦截器
request.interceptors.request.use(config => {
  if (config.download) {
    config.responseType = 'blob'
    config.timeout = 60000
  }
  return config
}, err => Promise.reject(err))

// 配置拦截器
request.interceptors.request.use(config => {
  if (!(config.url + config.baseURL)) {
    source.cancel('请求地址为空')
  }
  config.method = config._method
  if (config.headers['Content-Type'] === 'application/x-www-form-urlencoded') {
    config.data = toFormData(config.data)
  }
  return config
}, err => Promise.reject(err))

// rep拦截器
request.interceptors.response.use(rep => {
  NProgress.done()
  // 错误状态rep拦截器
  if (checkStatus(rep.config.status, rep.data.status)) {
    Message({
      type: 'error',
      message: `${rep.data.status} : ${rep.data.msg || '错误状态码'}`
    })
    return Promise.reject(rep)
  }
  // 下载rep拦截器
  if (rep.config.download) {
    const content = rep.data
    const blob = new Blob([content])
    const fileName = Date.now() + '.xls'
    if ('download' in document.createElement('a')) { // 非IE下载
      const elink = document.createElement('a')
      elink.download = fileName
      elink.style.display = 'none'
      elink.href = URL.createObjectURL(blob)
      document.body.appendChild(elink)
      elink.click()
      URL.revokeObjectURL(elink.href) // 释放URL 对象
      document.body.removeChild(elink)
    } else { // IE10+下载
      navigator.msSaveBlob(blob, fileName)
    }
  }
  return rep
}, err => {
  NProgress.done()
  // 下载rep拦截器
  if (err.config && err.config.download) {
    Notification({
      type: 'error',
      message: `下载失败`
    })
  } else if (err.code === 'ECONNABORTED' && err.message.indexOf('timeout') !== -1) {
    Message({
      type: 'error',
      message: `请求超时`
    })
  } else if (err.response) {
    Notification({
      type: 'error',
      message: `${err.response.data.msg || '请求失败'}`
    })
    // 错误状态rep拦截器
    // if (err.response.data.code === 700) {
    //   MessageBox.confirm('您已被登出，可以取消继续留在该页面，或者重新登录。', '确定登出', {
    //     confirmButtonText: '重新登录',
    //     cancelButtonText: '取消',
    //     type: 'warning'
    //   }).then(() => {
    //     store.dispatch('FedLogOut').then(() => {
    //       location.reload()// 为了重新实例化vue-router对象 避免bug
    //     })
    //   })
    //   return Promise.reject(err.response)
    // }
  } else if (err.message) {
    Message({
      type: 'error',
      message: `${err.message} 请求失败`
    })
  } else {
    Message({
      type: 'error',
      message: `请求失败`
    })
  }
  return Promise.reject(err)
})

export default request
