import cookie from 'js-cookie'
import { AESencrypt, AESdecrypt } from '@/utils/crypto'

// cookie
export function setCookie(key, plaintext, config) {
  if (key) {
    return cookie.set(AESencrypt(key), AESencrypt(plaintext instanceof Object ? JSON.stringify(plaintext) : plaintext), Object.assign({ expires: 0.5 }, config))
  }
}
export function getCookie(key) {
  if (key) {
    const ciphetext = cookie.get(AESencrypt(key))
    const plaintext = ciphetext && AESdecrypt(ciphetext)
    return plaintext && JSON.parse(plaintext)
  }
}
export function removeCookie(key) {
  if (key) {
    return cookie.remove(AESencrypt(key))
  }
}
export function clearCookie() {
  return document.cookie.split(';').map(e => cookie.remove(e.split('=')[0]))
}

// sessionStorage
export function setSession(key, plaintext) {
  if (key) {
    return sessionStorage.setItem(AESencrypt(key), AESencrypt(plaintext instanceof Object ? JSON.stringify(plaintext) : plaintext))
  }
}
export function getSession(key) {
  if (key) {
    const ciphetext = sessionStorage.getItem(AESencrypt(key))
    const plaintext = ciphetext && AESdecrypt(ciphetext)
    return plaintext && JSON.parse(plaintext)
  }
}
export function removeSession(key) {
  return sessionStorage.removeItem(AESencrypt(key))
}
export function clearSession() {
  return sessionStorage.clear()
}

// localStorage
export function setLocal(key, plaintext) {
  if (key) {
    return localStorage.setItem(AESencrypt(key), AESencrypt(plaintext instanceof Object ? JSON.stringify(plaintext) : plaintext))
  }
}
export function getLocal(key) {
  if (key) {
    const ciphetext = localStorage.getItem(AESencrypt(key))
    const plaintext = ciphetext && AESdecrypt(ciphetext)
    return plaintext && JSON.parse(plaintext)
  }
}
export function removeLocal(key) {
  return localStorage.removeItem(AESencrypt(key))
}
export function clearLocal() {
  return localStorage.clear()
}

export function clearAll() {
  clearCookie()
  localStorage.clear()
  sessionStorage.clear()
}

